@@ -22,16 +22,6 @@ XLINK_NAMESPACE = "http://www.w3.org/1999/xlink"
 XPATH_FUNCTIONS_NAMESPACE = "http://www.w3.org/2005/xpath-functions"
 XQT_ERRORS_NAMESPACE = "http://www.w3.org/2005/xqt-errors"
 
-XPATH_1_DEFAULT_NAMESPACES = {'xml': XML_NAMESPACE}
-
-XPATH_2_DEFAULT_NAMESPACES = {
-    'xml': XML_NAMESPACE,
-    'xs': XSD_NAMESPACE,
-    'xlink': XLINK_NAMESPACE,
-    'fn': XPATH_FUNCTIONS_NAMESPACE,
-    'err': XQT_ERRORS_NAMESPACE
-}
-
 # XML namespace attributes
 XML_BASE = '{%s}base' % XML_NAMESPACE
 XML_LANG = '{%s}lang' % XML_NAMESPACE
