@@ -756,7 +756,7 @@ def select(self, context=None):
         for elem in self[0].select(context):
             if not is_element_node(elem):
                 self.wrong_type("left operand must returns element nodes: %r" % elem)
-            for _ in context.iter_descendants(item=elem, axis='descendant-or-self'):
+            for _ in context.iter_descendants(item=elem):
                 for result in self[1].select(context):
                     yield result
 
@@ -1071,12 +1071,12 @@ def evaluate(self, context=None):
                    for k in range(len(self)))
 
 
-@method(function('string-length', nargs=1))
+@method(function('string-length', nargs=(0, 1)))
 def evaluate(self, context=None):
     return len(self.get_argument(context, default_to_context=True, default='', cls=string_base_type))
 
 
-@method(function('normalize-space', nargs=1))
+@method(function('normalize-space', nargs=(0, 1)))
 def evaluate(self, context=None):
     if self.parser.version == '1.0':
         arg = self.string_value(self.get_argument(context, default_to_context=True, default=''))
