@@ -21,12 +21,14 @@ import locale
 import unicodedata
 import xml.etree.ElementTree as ElementTree
 
-from .compat import PY3, string_base_type, unicode_chr, urlparse, urljoin, urllib_quote, unicode_type
+from .compat import PY3, string_base_type, unicode_chr, urlparse, urljoin, \
+    urllib_quote, unicode_type, URLError
 from .datatypes import QNAME_PATTERN, DateTime10, Date10, Time, Timezone, Duration, DayTimeDuration
-from .namespaces import prefixed_to_qname, get_namespace
-from .xpath_context import XPathSchemaContext
-from .xpath_nodes import is_document_node, is_xpath_node, is_element_node, \
-    is_attribute_node, node_name, node_nilled, node_base_uri, node_document_uri
+from .namespaces import prefixed_to_qname, get_namespace, XML_ID
+from .xpath_context import XPathContext, XPathSchemaContext
+from .xpath_nodes import AttributeNode, is_document_node, is_xpath_node, is_element_node, \
+    is_attribute_node, node_name, node_is_id, node_is_idrefs, node_nilled, node_base_uri, \
+    node_document_uri, node_kind, etree_deep_equal
 from .xpath2_parser import XPath2Parser
 
 method = XPath2Parser.method
@@ -36,8 +38,26 @@ WRONG_REPLACEMENT_PATTERN = re.compile(r'(?<!\\)\$([^\d]|$)|((?<=[^\\])|^)\\([^$
 
 
 ###
-# Node types
-@method(function('document-node', nargs=(0, 1)))
+# Sequence types (allowed only for type checking in treat-as/instance-of statements)
+@method(function('empty-sequence', nargs=0, label='sequence type'))
+def evaluate(self, context=None):
+    if context is not None:
+        return isinstance(context.item, list) and not context.item
+
+
+@method(function('item', nargs=0, label='sequence type'))
+def evaluate(self, context=None):
+    if context is None:
+        return
+    elif context.item is None:
+        return context.root
+    else:
+        return context.item
+
+
+###
+# Kind tests (sequence types that can appear also in XPath expressions)
+@method(function('document-node', nargs=(0, 1), label='kind test'))
 def evaluate(self, context=None):
     if context is not None:
         if context.item is None and is_document_node(context.root):
@@ -47,17 +67,22 @@ def evaluate(self, context=None):
                 return context.root
 
 
-@method(function('element', nargs=(0, 2)))
+@method(function('element', nargs=(0, 2), label='kind test'))
 def evaluate(self, context=None):
     if context is not None:
         if not self:
             if is_element_node(context.item):
                 return context.item
-        elif is_element_node(context.item, self[1].evaluate(context)):
-            return context.item
+        elif is_element_node(context.item, self[0].evaluate(context)):
+            if len(self) == 1:
+                return context.item
+            elif self.xsd_type is not None:
+                type_annotation = self[1].evaluate(context)
+                if self.xsd_type.is_matching(type_annotation, self.parser.default_namespace):
+                    return context.item
 
 
-@method(function('schema-attribute', nargs=1))
+@method(function('schema-attribute', nargs=1, label='kind test'))
 def evaluate(self, context=None):
     attribute_name = self[0].source
     qname = prefixed_to_qname(attribute_name, self.parser.namespaces)
@@ -69,7 +94,7 @@ def evaluate(self, context=None):
             return context.item
 
 
-@method(function('schema-element', nargs=1))
+@method(function('schema-element', nargs=1, label='kind test'))
 def evaluate(self, context=None):
     element_name = self[0].source
     qname = prefixed_to_qname(element_name, self.parser.namespaces)
@@ -82,17 +107,10 @@ def evaluate(self, context=None):
             return context.item
 
 
-@method(function('empty-sequence', nargs=0))
-def evaluate(self, context=None):
-    if context is not None:
-        return isinstance(context.item, list) and not context.item
-
-
 @method('document-node')
 @method('element')
 @method('schema-attribute')
 @method('schema-element')
-@method('empty-sequence')
 def select(self, context=None):
     if context is not None:
         for _ in context.iter_children_or_self():
@@ -136,9 +154,6 @@ def evaluate(self, context=None):
         return []
     elif not isinstance(qname, string_base_type):
         raise self.error('FORG0006', 'argument has an invalid type %r' % type(qname))
-    elif not qname:
-        return ''
-
     match = QNAME_PATTERN.match(qname)
     if match is None:
         raise self.error('FOCA0002', 'argument must be an xs:QName')
@@ -195,7 +210,8 @@ def select(self, context=None):
         else:
             # For ElementTree returns module registered prefixes
             prefixes = {x for x in self.parser.namespaces}
-            prefixes.update(x for x in ElementTree._namespace_map.values())
+            etree_nsmap = getattr(ElementTree, '_namespace_map', {})
+            prefixes.update(x for x in etree_nsmap.values())
             for prefix in prefixes:
                 yield prefix
 
@@ -203,6 +219,10 @@ def select(self, context=None):
 @method(function('resolve-QName', nargs=2))
 def evaluate(self, context=None):
     if context is not None:
+        elem = self.get_argument(context, index=1)
+        if not is_element_node(elem):
+            raise self.error('FORG0006', '2nd argument %r is not a node' % elem)
+
         qname = self.get_argument(context.copy())
         if qname is None:
             return []
@@ -213,9 +233,6 @@ def evaluate(self, context=None):
             raise self.error('FOCA0002', '1st argument must be an xs:QName')
         prefix = match.groupdict()['prefix'] or ''
 
-        elem = self.get_argument(context, index=1)
-        if not is_element_node(elem):
-            raise self.error('FORG0006', '2nd argument %r is not a node' % elem)
         ns_uris = {get_namespace(e.tag) for e in elem.iter()}
         for p, uri in self.parser.namespaces.items():
             if uri in ns_uris:
@@ -228,18 +245,6 @@ def evaluate(self, context=None):
             raise self.error('FONS0004', 'No namespace found for prefix %r' % prefix)
 
 
-###
-# Context item
-@method(function('item', nargs=0))
-def evaluate(self, context=None):
-    if context is None:
-        return
-    elif context.item is None:
-        return context.root
-    else:
-        return context.item
-
-
 ###
 # Accessor functions
 @method(function('node-name', nargs=1))
@@ -505,6 +510,46 @@ def select(self, context=None):
             raise self.error('FORG0005')
 
 
+###
+# Comparing sequences
+@method(function('deep-equal', nargs=(2, 3)))
+def evaluate(self, context=None):
+
+    def deep_equal():
+        while True:
+            value1 = next(seq1, None)
+            value2 = next(seq2, None)
+            if (value1 is None) ^ (value2 is None):
+                return False
+            elif value1 is None:
+                return True
+            elif (is_xpath_node(value1)) ^ (is_xpath_node(value2)):
+                return False
+            elif not is_xpath_node(value1):
+                if value1 != value2:
+                    return False
+            elif node_kind(value1) != node_kind(value2):
+                return False
+            elif not is_element_node(value1):
+                if value1 != value2:
+                    return False
+            elif not etree_deep_equal(value1, value2):
+                return False
+
+    if context is None:
+        seq1 = iter(self[0].select())
+        seq2 = iter(self[1].select())
+    else:
+        seq1 = iter(self[0].select(context.copy()))
+        seq2 = iter(self[1].select(context.copy()))
+
+    if len(self) > 2:
+        with self.use_locale(collation=self.get_argument(context, 2)):
+            return deep_equal()
+    else:
+        return deep_equal()
+
+
 ###
 # Regex
 @method(function('matches', nargs=(2, 3)))
@@ -915,7 +960,20 @@ def evaluate(self, context=None):
 
 
 ###
-# Context functions
+# Static context functions
+@method(function('default-collation', nargs=0))
+def evaluate(self, context=None):
+    return self.parser.default_collation
+
+
+@method(function('static-base-uri', nargs=0))
+def evaluate(self, context=None):
+    if self.parser.base_uri is not None:
+        return self.parser.base_uri
+
+
+###
+# Dynamic context functions
 @method(function('current-dateTime', nargs=0))
 def evaluate(self, context=None):
     dt = datetime.datetime.now() if context is None else context.current_dt
@@ -942,29 +1000,134 @@ def evaluate(self, context=None):
         return Timezone(datetime.timedelta(seconds=time.timezone))
 
 
-@method(function('static-base-uri', nargs=0))
-def evaluate(self, context=None):
-    if self.parser.base_uri is not None:
-        return self.parser.base_uri
-
-
 ###
 # The root function (Ref: https://www.w3.org/TR/2010/REC-xpath-functions-20101214/#func-root)
 @method(function('root', nargs=(0, 1)))
 def evaluate(self, context=None):
-    if self:
-        item = self.get_argument(context)
-    elif context is None:
+    if context is None:
         raise self.error('XPDY0002')
+    elif isinstance(context, XPathSchemaContext):
+        return
+    elif not self:
+        if context.item is None or is_xpath_node(context.item):
+            return context.root
+        else:
+            raise self.error('XPTY0004')
     else:
-        item = context.item
+        item = self.get_argument(context)
+        if not is_xpath_node(item):
+            raise self.error('XPTY0004')
+        elif any(item is x for x in context.iter()):
+            return context.root
 
-    if item is None:
-        return []
-    elif is_xpath_node(item):
-        return item
+        for uri, doc in context.documents.items():
+            doc_context = XPathContext(root=doc)
+            if any(item is x for x in doc_context.iter()):
+                return doc
+
+
+###
+# Functions that generate sequences
+XPath2Parser.duplicate('id', 'element-with-id')  # To preserve backwards compatibility
+XPath2Parser.unregister('id')
+
+
+@method(function('id', nargs=(1, 2)))
+def select(self, context=None):
+    # TODO: PSVI bindings with also xsi:type evaluation
+    idrefs = list(self[0].select(None if context is None else context.copy()))
+    node = self.get_argument(context, index=1, default_to_context=True)
+    if context is None or node is not context.item:
+        if not is_document_node(node):
+            raise self.error('FODC0001', 'cannot retrieve document root')
+        root = node
     else:
-        raise self.error('XPTY0004')
+        if not is_document_node(context.root):
+            raise self.error('FODC0001')
+        elif not is_xpath_node(node):
+            raise self.error('XPTY0004')
+        root = context.root
+
+    for elem in root.iter():
+        if node_is_id(elem) and any(v == elem.text for x in idrefs for v in x.split()):
+            yield elem
+            continue
+        for attr in map(lambda x: AttributeNode(*x), elem.attrib.items()):
+            if any(v == attr.value for x in idrefs for v in x.split()):
+                yield elem
+                break
+
+
+@method(function('idref', nargs=(1, 2)))
+def select(self, context=None):
+    # TODO: PSVI bindings with also xsi:type evaluation
+    ids = list(self[0].select(None if context is None else context.copy()))
+    node = self.get_argument(context, index=1, default_to_context=True)
+    if context is None or node is not context.item:
+        if not is_document_node(node):
+            raise self.error('FODC0001', 'cannot retrieve document root')
+        root = node
+    else:
+        if not is_document_node(context.root):
+            raise self.error('FODC0001')
+        elif not is_xpath_node(node):
+            raise self.error('XPTY0004')
+        root = context.root
+
+    for elem in root.iter():
+        if node_is_idrefs(elem) and any(v in elem.text.split() for x in ids for v in x.split()):
+            yield elem
+            continue
+        for attr in map(lambda x: AttributeNode(*x), elem.attrib.items()):
+            if attr.name != XML_ID and any(v in attr.value.split() for x in ids for v in x.split()):
+                yield elem
+                break
+
+
+@method(function('doc', nargs=1))
+@method(function('doc-available', nargs=1))
+def evaluate(self, context=None):
+    uri = self.get_argument(context)
+    if uri is None:
+        return None if self.symbol == 'doc' else False
+
+    try:
+        uri = self.get_absolute_uri(uri)
+    except URLError:
+        raise self.error('FODC0005')
+
+    if context is not None and not isinstance(context, XPathSchemaContext):
+        try:
+            doc = context.documents[uri]
+        except KeyError:
+            if self.symbol == 'doc':
+                raise self.error('FODC0005')
+            return False
+        else:
+            if not is_document_node(doc):
+                raise self.error('FODC0005')
+            return doc if self.symbol == 'doc' else True
+
+
+@method(function('collection', nargs=(0, 1)))
+def evaluate(self, context=None):
+    uri = self.get_argument(context)
+    if context is None or isinstance(context, XPathSchemaContext):
+        return
+    elif not self or uri is None:
+        if context.default_collection is None:
+            raise self.error('FODC0002', 'no default collection has been defined')
+        return context.default_collection
+
+    try:
+        uri = self.get_absolute_uri(uri)
+    except URLError:
+        raise self.error('FODC0004')
+
+    try:
+        return self.parser.collections[uri]
+    except KeyError:
+        raise self.error('FODC0004', '{!r} collection not found'.format(uri))
 
 
 ###
