@@ -15,15 +15,17 @@ from __future__ import division
 from itertools import product
 from abc import ABCMeta
 import decimal
+import locale
 import math
 import operator
 
 from .compat import MutableSequence, urlparse, unicode_type
 from .exceptions import ElementPathError, ElementPathKeyError, \
     ElementPathTypeError, MissingContextError
-from .namespaces import XSD_NAMESPACE, XPATH_FUNCTIONS_NAMESPACE, \
-    XPATH_2_DEFAULT_NAMESPACES, XSD_NOTATION, XSD_ANY_ATOMIC_TYPE, get_namespace, \
-    qname_to_prefixed, prefixed_to_qname, XSD_UNTYPED_ATOMIC
+from .namespaces import XSD_NAMESPACE, XML_NAMESPACE, XLINK_NAMESPACE, \
+    XPATH_FUNCTIONS_NAMESPACE, XQT_ERRORS_NAMESPACE, XSD_NOTATION, \
+    XSD_ANY_ATOMIC_TYPE, get_namespace, qname_to_prefixed, prefixed_to_qname, \
+    XSD_UNTYPED_ATOMIC
 from .datatypes import UntypedAtomic, XSD_BUILTIN_TYPES
 from .xpath_nodes import is_xpath_node
 from .xpath1_parser import XPath1Parser
@@ -97,7 +99,7 @@ class XPath2Parser(XPath1Parser):
         'zero-or-one', 'one-or-more', 'exactly-one',
 
         # Comparing function for sequences
-        # TODO 'deep-equal',
+        'deep-equal',
 
         # Pattern matching functions
         'matches', 'replace', 'tokenize',
@@ -126,8 +128,11 @@ class XPath2Parser(XPath1Parser):
         'QName', 'local-name-from-QName', 'prefix-from-QName', 'local-name-from-QName',
         'namespace-uri-from-QName', 'namespace-uri-for-prefix', 'in-scope-prefixes', 'resolve-QName',
 
-        # Context functions (TODO: 'default-collation')
-        'current-dateTime', 'current-date', 'current-time', 'implicit-timezone', 'static-base-uri',
+        # Static context functions
+        'default-collation', 'static-base-uri',
+
+        # Dynamic context functions
+        'current-dateTime', 'current-date', 'current-time', 'implicit-timezone',
 
         # Node set functions
         'root',
@@ -141,21 +146,25 @@ class XPath2Parser(XPath1Parser):
         'float', 'nonNegativeInteger', 'positiveInteger', 'nonPositiveInteger', 'negativeInteger',
         'unsignedLong', 'unsignedInt', 'unsignedShort', 'unsignedByte', 'dateTime', 'date', 'time',
         'gDay', 'gMonth', 'gYear', 'gMonthDay', 'gYearMonth', 'duration', 'dayTimeDuration',
-        'yearMonthDuration', 'base64Binary', 'hexBinary'
+        'yearMonthDuration', 'base64Binary', 'hexBinary',
 
-        # TODO: Functions and Operators that Generate Sequences
-        # 'id', 'idref', 'doc', 'doc-available', 'collection',
+        # Functions and Operators that Generate Sequences ('id' changes but is already registered)
+        'element-with-id', 'idref', 'doc', 'doc-available', 'collection',
     }
 
-    QUALIFIED_FUNCTIONS = {
-        'attribute', 'comment', 'document-node', 'element', 'empty-sequence', 'if', 'item', 'node',
-        'processing-instruction', 'schema-attribute', 'schema-element', 'text', 'typeswitch'
+    DEFAULT_NAMESPACES = {
+        'xml': XML_NAMESPACE,
+        'xs': XSD_NAMESPACE,
+        'xlink': XLINK_NAMESPACE,
+        'fn': XPATH_FUNCTIONS_NAMESPACE,
+        'err': XQT_ERRORS_NAMESPACE
     }
 
-    DEFAULT_NAMESPACES = XPATH_2_DEFAULT_NAMESPACES
+    PATH_STEP_LABELS = ('axis', 'function', 'kind test')
 
-    def __init__(self, namespaces=None, variables=None, strict=True, default_namespace=None,
-                 function_namespace=None, schema=None, base_uri=None, compatibility_mode=False):
+    def __init__(self, namespaces=None, variables=None, strict=True, compatibility_mode=False,
+                 default_namespace=None, function_namespace=None, schema=None, base_uri=None,
+                 default_collation=None, documents=None, collections=None, default_collection=None):
         super(XPath2Parser, self).__init__(namespaces, variables, strict)
         if default_namespace is not None:
             self.namespaces[''] = default_namespace
@@ -174,6 +183,10 @@ class XPath2Parser(XPath1Parser):
 
         self.base_uri = None if base_uri is None else urlparse(base_uri).geturl()
         self._compatibility_mode = compatibility_mode
+        self._default_collation = default_collation
+        self.documents = {} if documents is None else dict(documents)
+        self.collections = {} if collections is None else dict(collections)
+        self.default_collection = default_collection
 
     def __getstate__(self):
         state = self.__dict__.copy()
@@ -193,6 +206,13 @@ class XPath2Parser(XPath1Parser):
     def compatibility_mode(self, value):
         self._compatibility_mode = value
 
+    @property
+    def default_collation(self):
+        if self._default_collation is not None:
+            return self._default_collation
+        default_locale = locale.getdefaultlocale()
+        return '.'.join(default_locale) if default_locale[1] else default_locale[0]
+
     @property
     def default_namespace(self):
         return self.namespaces.get('')
@@ -315,17 +335,6 @@ class XPath2Parser(XPath1Parser):
         self.symbol_table[symbol] = token_class
         return token_class
 
-    def next_is_path_step_token(self):
-        return self.next_token.label in ('axis', 'function') or self.next_token.symbol in {
-            '(integer)', '(string)', '(float)', '(decimal)', '(name)', '*', '@', '..', '.', '(', '{'
-        }
-
-    def next_is_sequence_type_token(self):
-        return self.next_token.symbol in {
-            '(name)', ':', 'empty-sequence', 'item', 'document-node', 'element', 'attribute',
-            'text', 'comment', 'processing-instruction', 'schema-attribute', 'schema-element'
-        }
-
     def is_instance(self, obj, type_qname):
         if type_qname == XSD_UNTYPED_ATOMIC:
             return isinstance(obj, UntypedAtomic)
@@ -343,6 +352,9 @@ class XPath2Parser(XPath1Parser):
 
     def parse(self, source):
         root_token = super(XPath1Parser, self).parse(source)
+        if root_token.label == 'sequence type':
+            raise root_token.error('XPST0003', message="not allowed in XPath expression")
+
         if self.schema is None:
             try:
                 root_token.evaluate()  # Static context evaluation
@@ -517,8 +529,11 @@ def select(self, context=None):
 @method('treat', bp=61)
 def led(self, left):
     self.parser.advance('of' if self.symbol == 'instance' else 'as')
-    if not self.parser.next_is_sequence_type_token():
-        self.parser.next_token.wrong_syntax()
+    next_token = self.parser.next_token
+    if next_token.symbol not in ('(name)', ':') and \
+            next_token.label not in ('kind test', 'sequence type'):
+        next_token.wrong_syntax()
+
     self[:] = left, self.parser.expression(rbp=self.rbp)
     next_symbol = self.parser.next_token.symbol
     if self[1].symbol != 'empty-sequence' and next_symbol in ('?', '*', '+'):
@@ -535,7 +550,7 @@ def evaluate(self, context=None):
         for _ in self[0].select(context):
             return False
         return True
-    elif self[1].label == 'function':
+    elif self[1].label in ('kind test', 'sequence type'):
         for position, item in enumerate(self[0].select(context)):
             if self[1].evaluate(context) is None:
                 return False
@@ -566,7 +581,7 @@ def evaluate(self, context=None):
     if self[1].symbol == 'empty-sequence':
         for _ in self[0].select(context):
             self.wrong_sequence_type()
-    elif self[1].label == 'function':
+    elif self[1].label in ('kind test', 'sequence type'):
         for position, item in enumerate(self[0].select(context)):
             if self[1].evaluate(context) is None:
                 if context is not None and not isinstance(context, XPathSchemaContext):
