@@ -24,7 +24,7 @@ import locale
 import contextlib
 from decimal import Decimal
 
-from .compat import string_base_type, unicode_type
+from .compat import string_base_type, unicode_type, urljoin, urlparse, uses_relative
 from .exceptions import xpath_error
 from .namespaces import XQT_ERRORS_NAMESPACE
 from .xpath_nodes import AttributeNode, TypedAttribute, TypedElement, \
@@ -37,6 +37,9 @@ from .tdop_parser import Token
 from .xpath_context import XPathSchemaContext
 
 
+UNICODE_CODEPOINT_COLLATION = "http://www.w3.org/2005/xpath-functions/collation/codepoint"
+
+
 def ordinal(n):
     if n in {11, 12, 13}:
         return '%dth' % n
@@ -87,10 +90,8 @@ class XPathToken(Token):
             return '$%s variable reference' % (self[0].value if self else '')
         elif symbol == ',':
             return 'comma operator' if self.parser.version > '1.0' else 'comma symbol'
-        elif label == 'function':
-            return '%r function' % symbol
-        elif label == 'axis':
-            return '%r axis' % symbol
+        elif label in ('axis', 'function', 'sequence type', 'kind test', 'constructor'):
+            return '%r %s' % (symbol, label)
         return super(XPathToken, self).__str__()
 
     @property
@@ -98,7 +99,7 @@ class XPathToken(Token):
         symbol, label = self.symbol, self.label
         if label == 'axis':
             return '%s::%s' % (self.symbol, self[0].source)
-        elif label in ('function', 'constructor'):
+        elif label in ('function', 'sequence type', 'kind test', 'constructor'):
             return '%s(%s)' % (self.symbol, ', '.join(item.source for item in self))
         elif symbol == ':':
             return '%s:%s' % (self[0].source, self[1].source)
@@ -349,6 +350,21 @@ class XPathToken(Token):
 
         return arg1, arg2
 
+    def get_absolute_uri(self, uri):
+        """
+        Obtains an absolute URI from the argument and the static context.
+
+        :param uri: a string representing an URI.
+        :returns: the argument if it's an absolute URI. Otherwise returns the URI
+        obtained by the join o the base_uri of the static context with the
+        argument. Returns the argument if the base_uri is `None'.
+        """
+        url_parts = urlparse(uri)
+        if url_parts.scheme not in uses_relative or url_parts.path.startswith('/') \
+                or self.parser.base_uri is None:
+            return uri
+        return urljoin(self.parser.base_uri, uri)
+
     def adjust_datetime(self, context, cls):
         """
         XSD datetime adjust function helper.
@@ -460,8 +476,9 @@ class XPathToken(Token):
     @contextlib.contextmanager
     def use_locale(self, collation):
         """A context manager for setting a specific collation for a code block."""
-        locale.setlocale(locale.LC_ALL, '')
-        default_locale = locale.getlocale()
+        loc = locale.getlocale()
+        if collation == UNICODE_CODEPOINT_COLLATION:
+            collation = 'en_US.UTF-8'
 
         try:
             locale.setlocale(locale.LC_ALL, collation)
@@ -470,7 +487,7 @@ class XPathToken(Token):
         else:
             yield
         finally:
-            locale.setlocale(locale.LC_ALL, default_locale)
+            locale.setlocale(locale.LC_ALL, loc)
 
     ###
     # XPath data accessors base functions
